@@ -330,6 +330,9 @@ class DDPPlugin(ParallelPlugin):
             )
 
     def pre_dispatch(self):
+        # share ddp pids to all processes
+        self._share_information_to_prevent_deadlock()
+
         # move the model to the correct device
         self.model_to_device()
 
@@ -338,9 +341,6 @@ class DDPPlugin(ParallelPlugin):
 
         self.configure_ddp()
 
-        # share ddp pids to all processes
-        self._share_information_to_prevent_deadlock()
-
     def post_dispatch(self) -> None:
         self.cluster_environment.teardown()
 
@@ -436,6 +436,10 @@ class DDPPlugin(ParallelPlugin):
 
         sync_dir = self._sync_dir
 
+        if not sync_dir:
+            rank_zero_warn("Error handling mechanism for deadlock detection is uninitialized. Skipping check.")
+            return
+
         # The cluster may be configured to periodically purge the `/tmp`
         # directory, in which case `sync_dir` may not exist anymore at this
         # point. Idempotently create it to ensure its existence.
